package classes;

import java.util.ArrayList;
import java.util.HashMap;

import ui.controller.MultiPurposeController;
import ui.model.idCounter;

public class HolonObject extends CpsObject {

	/* Array of all consumers */
	private ArrayList<HolonElement> elements;
	/* Total of consumption */
	private float currentEnergy;
	private HashMap<String, Integer> EleIdx;
	/**
	 * State of the building: 0 = fully supplied (currentEnergy == 0) 1 = not
	 * enough energy (currentEnergy > 0) 2 = oversupplied (currentEnergy < 0)
	 */
	int state;

	/**
	 * Constructor Set by default the name of the object equals to the category
	 * name, until the user changes it.
	 */
	public HolonObject(String ObjName) {
		super(ObjName);
		setEleIdx(new HashMap<>());
		setElements(new ArrayList<HolonElement>());

	}

	public HolonObject(String ObjName, String obj) {
		super(ObjName);
		super.setName(obj);
		setEleIdx(new HashMap<>());
		setElements(new ArrayList<HolonElement>());
	}

	public HolonObject(CpsObject obj) {
		super(obj);
		setEleIdx(MultiPurposeController.copyHashMap(((HolonObject) obj).getEleIdx()));
//		setElements(((HolonObject) obj).getCopyofElements());
		setElements(MultiPurposeController.copyArrayList(((HolonObject)obj).getElements()));
	}

	/**
	 * @return the elements
	 */
	public ArrayList<HolonElement> getElements() {
		return elements;
	}

	/**
	 * @param elements
	 *            the elements to set
	 */
	public void setElements(ArrayList<HolonElement> elements) {
		this.elements = elements;
	}

	public void addElements(HolonElement element) {
		elements.add(element);
	}

	/**
	 * @return the currentEnergy
	 */
	public float getCurrentEnergy() {
		float temp = 0;
		for (HolonElement e : getElements()) {
			if (e.getActive()) {
				temp = temp + e.getTotalEnergy();
			}
		}
		currentEnergy = temp;
		return currentEnergy;
	}

	/**
	 * @param currentEnergy
	 *            the currentEnergy to set
	 */
	public void setCurrentEnergy(float currentEnergy) {
		this.currentEnergy = currentEnergy;
	}

	/**
	 * deletes Element
	 * 
	 * @param idx
	 */
	public void deleteElement(int idx) {
		elements.remove(idx);
	}

	public float calculateCurrentEnergy() {
		return currentEnergy;
	}

	/**
	 * String of all consumers in this HolonObject
	 * 
	 * @return all the names of this HolonObject separated by "," each object
	 */
	public String toStringElements() {
		String objString = "Empty";
		for (HolonElement e : elements) {
			if (objString == "Empty") {
				objString = e.getEleName();
			} else {
				objString = objString + ", " + e.getEleName();
			}
		}
		return objString;
	}

	/**
	 * Copys the ArrayList
	 * 
	 * @param the
	 *            ArrayList to Copy
	 * 
	 * @return the currentEnergy
	 */
	public ArrayList<HolonElement> getCopyofElements() {
		ArrayList<HolonElement> temp = new ArrayList<>();
		for (HolonElement h : elements) {
			HolonElement he = new HolonElement(h);
			temp.add(he);
		}
		return temp;
	}

	/**
	 * @return the eleIdx
	 */
	public HashMap<String, Integer> getEleIdx() {
		return EleIdx;
	}

	/**
	 * @param eleIdx the eleIdx to set
	 */
	public void setEleIdx(HashMap<String, Integer> eleIdx) {
		EleIdx = eleIdx;
	}
}